## 7/1/2020 change AUC calculation to from start of the first transient to 60 minutes afterwards
## install.packages('chron')
##  setwd('your-directory/')
library(chron)

dat$time <- chron(times=dat$time)

### function to convert time to minutes ########
toMinute <- function(time1){
  hours(chron(times=time1))*60+minutes(chron(times=time1))+(seconds(chron(times=time1))/60)
  
}

### function to get the mean base and max of each spike #####
get_baseMax <- function(up_down1,val,dataset=dat){
  dat<- dataset
  if (nrow(up_down1)==1){
    .start1 <- which(dat$time==up_down1[1,1])
    .start0 <- max(1,.start1-200)
    bases<- mean(val[1:.start1])
  } else {
    base1 <-mean(val[1:which(dat$time==up_down1[1,1])])
    bases <- apply(cbind(up_down1[-nrow(up_down1),2],up_down1[-1,1]),1,function(v){mean(val[which(dat$time==v[1]):which(dat$time==v[2])])})
    bases <- c(base1,bases)
  }
  maxs <- apply(up_down1,1,function(v) {max(val[which(dat$time==v[1]):which(dat$time==v[2])])})
  cbind(bases,maxs)
}

### function to calculate AUC lag.time after start of the first spike, with or without first spike
get.auc<- function(up_down=up_down1, dataset=dat, s1=s, lag.time='01:00:00',first.peak=F){
  dat<- dataset 
  s<- s1
  if (is.null(up_down)) {auc<-0;return(auc)}
  i<-1
  dat1 <- dat[chron(times=dat$time)>=chron(times=up_down[i,1]) & chron(times=dat$time)<= chron(times=up_down[i,2]),]
  auc1 <- sum(pmax(dat1[,s]-as.numeric(up_down[i,3]),rep(0,nrow(dat1))))
  if (first.peak) auc <- auc1 else auc <-0
  if (nrow(up_down)==1) {return(auc)} ### why setting it to zero when there is only one up-down before?
  if (nrow(up_down)>1) {
    
    for (i in 2:nrow(up_down)){
      ### if up of the current is after down of 1st+ lag time return auc
      if (chron(times=up_down[i,1]) >  chron(times=up_down[1,1])+chron(times=lag.time)) return(auc) 
      ### if up of the current is before down of 1st+ lag time return auc but down of the current is after
      if (chron(times=up_down[i,1]) < chron(times=up_down[1,1])+chron(times=lag.time) &
          chron(times=up_down[i,2]) > chron(times=up_down[1,1])+chron(times=lag.time)) {
        
        dat1 <- dat[chron(times=dat$time)>=chron(times=up_down[i,1]) & chron(times=dat$time)<= chron(times=up_down[1,1])+chron(times=lag.time),]
        auc <- auc+sum(pmax(dat1[,s]-as.numeric(up_down[i,3]),rep(0,nrow(dat1))))
        
        return(auc)
      } 
      ### if down of the current is before down of 1st+ lag time return auc
      if (chron(times=up_down[i,2]) <  chron(times=up_down[1,1])+chron(times=lag.time)) {
        dat1 <- dat[chron(times=dat$time)>=chron(times=up_down[i,1]) & chron(times=dat$time)<= chron(times=up_down[i,2]),]
        auc <- auc+sum(pmax(dat1[,s]-as.numeric(up_down[i,3]),rep(0,nrow(dat1))))
        
      }
    }
    return(auc)
  }
  
}

### function to paint the spikes until lag.time after start of the first spike,
paint.auc<- function(up_down=up_down1, dataset=dat, s1=s, lag.time='01:00:00',first.peak=F){
  dat<- dataset
  s<- s1
  if (is.null(up_down)) {return()}
  if (first.peak) { i<-1
  dat1 <- dat[chron(times=dat$time)>=chron(times=up_down[i,1]) & chron(times=dat$time)<= chron(times=up_down[i,2]),]
  for (k in 1:nrow(dat1))
    segments(chron(times=dat1$time[k]),dat1[k,s],chron(times=dat1$time[k]),as.numeric(up_down[1,3]),col='lightblue',lty=2)
  }
  
  if (nrow(up_down)==1) {return()}
  if (nrow(up_down)>1) {
    
    for (i in 2:nrow(up_down)){
      ### if up of the current is after down of 1st+ lag time return auc
      if (chron(times=up_down[i,1]) >  chron(times=up_down[1,1])+chron(times=lag.time)) return() 
      ### if up of the current is before down of 1st+ lag time return auc but down of the current is after
      if (chron(times=up_down[i,1]) < chron(times=up_down[1,1])+chron(times=lag.time) &
          chron(times=up_down[i,2]) > chron(times=up_down[1,1])+chron(times=lag.time)) {
        
        dat1 <- dat[chron(times=dat$time)>=chron(times=up_down[i,1]) & chron(times=dat$time)<= chron(times=up_down[1,1])+chron(times=lag.time),]
        for (k in 1:nrow(dat1))
          segments(chron(times=dat1$time[k]),dat1[k,s],chron(times=dat1$time[k]),as.numeric(up_down[i,3]),col='lightblue',lty=2)
        
        return()
      } 
      ### if down of the current is before down of 1st+ lag time return auc
      if (chron(times=up_down[i,2]) <  chron(times=up_down[1,1])+chron(times=lag.time)) {
        dat1 <- dat[chron(times=dat$time)>=chron(times=up_down[i,1]) & chron(times=dat$time)<= chron(times=up_down[i,2]),]
        for (k in 1:nrow(dat1))
          segments(chron(times=dat1$time[k]),dat1[k,s],chron(times=dat1$time[k]),as.numeric(up_down[i,3]),col='lightblue',lty=2)
        
      }
    }
    return()
  }
  
}

### main function to count oscillation. The program automatically determines where to draw the horizontal line to count the spikes and do other calculation
### Sometimes the function may have trouble finding a good place to draw the horizontal line, the function count.osc.drawline is recommended

count.oscillation<- function(dataset=dat, s=1,lag.time1='01:00:00',save.plot=F){
  win.size <- 200
  z.cutoff <- 2.5
  
  dat <- dataset
  dat$time <- chron(times=dat$time)
  
  up_down <- NULL
  .stop <- F
  t.start <- 1
  s<- s+2
  max.diff <- range(dat[-(1:2),s]-dat[-c(nrow(dat)-1,nrow(dat)),s])
  #####################################################################################################################
  ## Moving windows. First pass to estimate up points and down points, then use non-peak data to 
  ## estimate baseline and estimate the height of the last oscillation. Set the mid point as baseline+peak.height/2
  ## recount oscillation based on the new line
  #####################################################################################################################
  ### moving window to go through the data from time 0 to the end, each time move by win.size
  while (!.stop){
    t.end <- min(win.size+t.start,nrow(dat))
    if (2*win.size+t.start>nrow(dat)) {t.end <- nrow(dat);.stop <- T}
    dat1<- dat[t.start:t.end,]
    
    if (diff(range(dat1[,s]))> diff(range(dat[,s]))/10) {
      base.v<- mean(dat1[dat1[,s]<quantile(dat1[,s],0.75),s])
      mid.point0 <- (base.v+max(dat1[,s]))/2
      
      ups <- which(scale(dat1[-(1:2),s]-dat1[-c(nrow(dat1)-1,nrow(dat1)),s])> z.cutoff)
      downs <- which(scale(dat1[-(1:2),s]-dat1[-c(nrow(dat1)-1,nrow(dat1)),s])< -z.cutoff)
      
      
      if (length(ups)>0 & length(downs)>0) {
        up_down1 <- NULL
        down1 <- 1
        #  plot(dat1$time,dat1[,s],type='l',xlab='time',ylab='value')
        
        for (k in 1:length(ups)){
          if(ups[k]>down1){
            down1<- downs[which(downs>ups[k])][1]
            if (is.na(down1)) break else
              up_down <- rbind(up_down,c(as.character(dat1$time[ups[k]]),as.character(dat1$time[down1])))
            up_down1 <- rbind(up_down1,c(as.character(dat1$time[ups[k]]),as.character(dat1$time[down1])))
          } else t.start <- t.start+win.size 
        }
        if (!is.null(up_down1)) {if (is.na(down1))
          t.start <- t.start-which(dat1$time==up_down1[nrow(up_down1),1])-10+win.size else
            t.start <-which(as.character(dat$time)==up_down1[nrow(up_down1),2])+1} else if ( sum((dat1[,s]- mid.point0)/diff(range(dat1[,s]))>0.5)>0)
              t.start <- t.start-which( sum((dat1[,s]- mid.point0)/diff(range(dat1[,s]))>0.5))-10+win.size else t.start <- t.start+win.size 
      } else { up_down1 <- NULL;t.start<- t.start+win.size}
      
    } else { up_down1 <- NULL;t.start<- t.start+win.size }
    if (t.start> nrow(dat)) .stop <- T
  }
  
  ### refine mid.point
  if (!is.null(up_down)) n.osc <- nrow(up_down) else n.osc <- 0
  if (!is.null(up_down)) {
   
    ###redefine mid.point and get new up_down list###
    spike.height <- diff(range(dat[which(dat$time==up_down[nrow(up_down),1]):which(dat$time==up_down[nrow(up_down),2]),s]))
    
    ## get baseline height and define mid.point as half way between the baseline height and the spike height.
    if (nrow(up_down)>1) {
      base.height <- mean(dat[which(dat$time==up_down[nrow(up_down),1]):which(dat$time==up_down[nrow(up_down)-1,2]),s])
    } else {
      base.height <- mean(dat[which(dat$time==up_down[nrow(up_down),2]):nrow(dat),s])
    }
    mid.point0 <- base.height + spike.height*.5
    
    ups <- which(dat[,s]> mid.point0)
    downs <- which(dat[,s]< mid.point0)
    down1 <- 1
    up_down1 <- NULL
    if (length(ups)==0 | length(downs)==0) up_down1=NULL else {
      for (k in 1:length(ups)){
        if(ups[k]>down1){
          down1<- downs[which(downs>ups[k])][1]
          if (is.na(down1)) break else
            up_down1 <- rbind(up_down1,c(as.character(dat$time[ups[k]]),as.character(dat$time[down1])))
        }
      }
    }
    ###################################################  
    osc_len1 <-NA
    delay1 <- NA
    
    n.osc<-0
    time.first.last<- NA
    osc.per.10min<- NA
    meanPeak<- NA
    TIbO<- NA
    
    n.osc1<-0
    time.first.last1<- NA
    osc.per.10min1<- NA
    meanPeak1<- NA
    TIbO1<- NA
    
    n.osc2<-0
    time.first.last2<- NA
    osc.per.10min2<- NA
    meanPeak2<- NA
    TIbO2<- NA
    
    if (!is.null(up_down1)){
      
      ### remove "peaks" that are less than 50% above the base
      get_baseMax(up_down1  , dat[,s])->baseMax
      ### decide which peaks to remove based on baseline and maxp heights
      ## either peak/base line>1.5 or the difference between the peak and the previous baseline after adjusting 
      ## for change in baseline levels is at least 12% of the mean peak-baseline across all spikes >0.2
      if (nrow(baseMax)>1) .keep <- (baseMax[,2]-baseMax[,1]-c(0,baseMax[-1,1]-baseMax[-nrow(baseMax),1]))/mean(baseMax[,2]-baseMax[,1])>0.12 else .keep <- T
      .keep <- .keep | baseMax[,2]/baseMax[,1] >=1.5
      up_down1 <- up_down1[.keep,,drop=F]
      if (nrow(up_down1)==0) up_down1 <- NULL
    }
    
    if (!is.null(up_down1)){
      n.osc <- nrow(up_down1) 
      
      #########################################
      
      delay1 <- toMinute(up_down1[1,1])
      osc_len <- apply(up_down1,1,function(v){toMinute(chron(times=v[2])-chron(times=v[1]))})
      osc_len1 <- osc_len[1]
      
      osc_len.rest <- mean(osc_len[-1])
      
      get_baseMax(up_down1  , dat[,s])->baseMax
      meanPeak <- mean(baseMax[,2]-baseMax[,1])
      ## all
      n.osc <- nrow(up_down1)
      
      if (n.osc >1) {
        time.first.last <-  toMinute(chron(times=up_down1[nrow(up_down1),2])-chron(times=up_down1[1,2]))
        osc.per.10min <-n.osc/time.first.last*10
        delays <- apply(cbind(up_down1[-nrow(up_down1),2],up_down1[-1,1]),1,function(v){toMinute(chron(times=v[2])-chron(times=v[1]))})
      }  else {
        time.first.last <-0
        osc.per.10min <-n.osc/10
        delays <-NA
      }
      if (length(delays)>2)
        TIbO <- coef(lm(1:length(delays)~delays))[2]
      
      ## 1 hour after 1st osc
      up_down2 <- up_down1[which( hours(chron(times=up_down1[,1])-chron(times=up_down1[1,1]))==0),,drop=F]
      n.osc1 <- nrow(up_down2)
      
      if (n.osc1>0) {get_baseMax(up_down2  , dat[,s])->baseMax1
        meanPeak1 <- mean(baseMax1[,2]-baseMax1[,1])
        
        if (n.osc1 >1) {
          time.first.last1 <-  toMinute(chron(times=up_down2[nrow(up_down2),2])-chron(times=up_down2[1,2]))
          osc.per.10min1 <-n.osc1/time.first.last1*10
          delays1 <- apply(cbind(up_down2[-nrow(up_down2),2],up_down2[-1,1]),1,function(v){toMinute(chron(times=v[2])-chron(times=v[1]))})
        } else {
          time.first.last1 <-0
          osc.per.10min1 <-n.osc/10
          delays1 <-NA
        }}
      
      if (length(delays1)>2)
        TIbO1 <- coef(lm(1:length(delays1)~delays1))[2]
      
      ## 2 hours
      up_down2 <- up_down1[which( hours(chron(times=up_down1[,1])-chron(times=up_down1[1,1]))<=1),,drop=F]
      n.osc2 <- nrow(up_down2)
      
      if (n.osc2>0) {get_baseMax(up_down2  , dat[,s])->baseMax2
        meanPeak2 <- mean(baseMax2[,2]-baseMax2[,1])
        
        if (n.osc2 >1) {
          time.first.last2 <-  toMinute(chron(times=up_down2[nrow(up_down2),2])-chron(times=up_down2[1,2]))
          osc.per.10min2 <-n.osc2/time.first.last2*10
          delays2 <- apply(cbind(up_down2[-nrow(up_down2),2],up_down2[-1,1]),1,function(v){toMinute(chron(times=v[2])-chron(times=v[1]))})
        } else {
          time.first.last2 <-0
          osc.per.10min2 <-n.osc/10
          delays2 <-NA
        }}
      
      if (length(delays2)>2)
        TIbO2 <- coef(lm(1:length(delays2)~delays2))[2]
    }
    
  } else up_down1<- NULL
  
  
  if (!is.null(up_down1)) {
    up_down1<- cbind(up_down1, baseMax)
    up_down1 <- cbind(up_down1,rep(mid.point0,nrow(up_down1)))
    colnames(up_down1) <- c('up.time','down.time','base.height','max.height','mid.point')
    
    #### get AUC #########
    up_down.accu <- up_down1
    time.s <- chron(times=dat$time)
    up.accu <- NULL
    down.accu <- NULL
    osc_len1.accu <- NULL
    
    for (t in 1:nrow(up_down1)) {
      if (t<nrow(up_down1)) start.next <- chron(times=up_down1[t+1,1]) else start.next <- min(chron(times=up_down1[t,2])+chron(times="00:05:00"), chron(times=time.s[length(time.s)])) 
      #if (t<nrow(up_down1)) base1<- up_down1[t+1, 3] else base1<- up_down1[t, 3] 
      base1<- as.numeric(up_down1[t, 3] )+(as.numeric(up_down1[t,4])-as.numeric(up_down1[t, 3] ))*0.18
      if (t>1) end.last <- chron(times=up_down1[t-1,2]) else end.last <- max(time.s[1],chron(times=up_down1[1,1])-chron(times="00:05:00"))
      time.range <- which(time.s>=end.last & time.s< start.next)
      above.base <- dat[time.range,s]>base1
      
      longest.true <- function(torf,max.len=0) {
        if (length(torf)==1) {max.len <- max.len+1*torf; max.len} else {
          if (!torf[1]) max.len else  longest.true(torf[-1],max.len+1)}}
      
      
      runofT <- sapply(1:(length(above.base)-1),function(v) longest.true(above.base[-(1:v)]))
      if (sum(runofT)==0) runofT[1] <- T
      up_down.accu[t,1] <- as.character(dat$time[time.range[which.max(runofT)]])
      up_down.accu[t,2] <- as.character(dat$time[time.range[which.max(runofT)+max(runofT)-1]])
      up_down.accu[t,1] <-as.character(min(chron(time=up_down.accu[t,1]),chron(time=up_down1[t,1])))
      up_down.accu[t,2] <-as.character(max(chron(time=up_down.accu[t,2]),chron(time=up_down1[t,2])))
      
      up_down1[t,1] <- up_down.accu[t,1]
      up_down1[t,2] <- up_down.accu[t,2]
      
      
    }
    osc_len1.accu <- toMinute(chron(times=up_down1[1,2])-chron(times=up_down1[1,1]))
    
    mid.time <- (chron(times=up_down1[,2])+chron(times=up_down1[,1]))/2
    up_down1<- cbind(up_down1,as.character(mid.time))
    colnames(up_down1)[6] <- 'mid.time'
    #### get areas under the peaks ###
    auc <- 0
    step.min <- toMinute(chron(times=dat$time)[2]-chron(times=dat$time)[1])
    if (!is.null(up_down1)) {
      for (i in 1:nrow(up_down1)) {
        dat1 <- dat[chron(times=dat$time)>=chron(times=up_down1[i,1]) & chron(times=dat$time)<=chron(times=up_down1[i,2]),]
        auc <- auc+sum(pmax(dat1[,s]-as.numeric(up_down1[i,3]),rep(0,nrow(dat1))))
      }
      
    }
    auc <- round(auc*step.min,2)
    
    auc0 <- round(get.auc(s1=s,up_down=up_down1,lag.time=lag.time1,first.peak=F)*step.min,2)
    auc1 <- round(get.auc(s1=s,up_down=up_down1,lag.time=lag.time1,first.peak=T)*step.min,2)
    ######### plotting ######## 
    if (save.plot) pdf(file=paste('sample',s-2,'_plot.pdf',sep=''))
    plot(dat$time,dat[,s],type='l',xlab='time',ylab='value',main=paste('Sample',s-2,' # of Osc=',n.osc,'; 1st length=',chron(times=chron(times=up_down1[1,2])-chron(times=up_down1[1,1])),' AUC=',auc," & ",auc0," & ",auc1,sep=''))
    if (!is.null(up_down1)) abline(v=chron(times=up_down1[,6]),lty=2,col='grey')
    abline(h=mid.point0,col='red')
    abline(v=chron(times=up_down1[1,1])+chron(times=lag.time1),col='blue')
    paint.auc(up_down1,s1=s, lag.time=lag.time1,first.peak=T)
    points(dat$time,dat[,s],type='l')
    
    
    write.table(up_down1,row.names = F,quote=F,sep='\t',file=paste('sample',s-2,'.txt',sep=''))
  } else plot(dat$time,dat[,s],type='l',xlab='time',ylab='value',main=paste('Sample',s-2,' # of Osc=',n.osc,sep=''))
  
  if (save.plot) dev.off()
  
  if (is.null(up_down1)) {osc_len1<-NA; osc_len1.corr <-NA;osc_len1.accu <-NA;auc<-NA;auc1<-NA; auc0<-NA;delay1<-NA;n.osc<-NA;time.first.last<-NA;osc.per.10min<-NA;meanPeak<-NA;TIbO<-NA;
  n.osc1<-NA;time.first.last1<-NA;osc.per.10min1<-NA;meanPeak1<-NA;TIbO1<-NA;
  n.osc2<-NA;time.first.last2<-NA;osc.per.10min2<-NA;meanPeak2<-NA;TIbO2<-NA
  }
  
  if (!is.null(up_down1)) osc_len1.corr<- osc_len1 * (as.numeric( baseMax[1,2])/(baseMax[1,2]-mid.point0))^(1/3)
  
  rst <- c(osc_len1.accu ,auc,auc1, auc0,delay1,
                     n.osc,time.first.last,osc.per.10min,meanPeak,
                     n.osc1,time.first.last1,osc.per.10min1,meanPeak1,
                     n.osc2,time.first.last2,osc.per.10min2,meanPeak2)
  names(rst) <-c("length of 1st Osc",'Total AUC','AUC w 1st peak','AUC w/o 1st peak',"Delay b/f 1st",
                 "n.osc (all)","time.first.last (all)","osc.per.10min (all)","meanPeak (all)",
                 "n.osc (1hr)","time.first.last (1hr)","osc.per.10min (1hr)","meanPeak (1hr)",
                 "n.osc (2hr)","time.first.last (2hr)","osc.per.10min (2hr)","meanPeak (2hr)")
    as.data.frame(rst)
  
}

count.osc.drawline <- function(s=1,mid.point0,length.at.height=NA,at.time=NA,lag.time1='01:00:00',dataset=dat,
                               file1='summary.stat.onesample.txt',.append=F,save.plot=F) {
  win.size <- 200
  z.cutoff <- 2.5
  
  s=s+2
  length.point <- length.at.height
  rst<- NULL
  if (length(mid.point0)>1 & is.na(at.time[1])) stop("Please provide cutoff time!")
  if (length(mid.point0)>1 & length(mid.point0)!=length(at.time)+1) stop("Length of midpoints should one longer the length of at.time")
  dat <- dataset
  #### multiple mid.point0
  up_down.all <- NULL
  if (!is.na(at.time[1]) & length(mid.point0)>1) bounds <- c(min(dat$time),at.time,max(dat$time)) else  bounds <- c(min(dat$time),max(dat$time))
  auc.all <- 0
  for (l in 1:length(mid.point0)) {
    ## get areas under the peaks ##   
    start.row <- which((dat$time)>=bounds[l])[1]
    end.row <- which((dat$time)>bounds[l+1])[1]
    if (is.na(end.row)) end.row <- nrow(dat)
    dat1 <- dat[start.row:end.row,]
    ups <- which(dat1[,s]> mid.point0[l])
    downs <- which(dat1[,s]< mid.point0[l])
    down1 <- 1
    
    up_down1 <- NULL
    if (length(ups)>0) {
      for (k in 1:length(ups)){
        if(ups[k]>down1){
          down1<- downs[which(downs>ups[k])][1]
          if (is.na(down1)) break else {#cat(k,ups[k],down1,'\n')
            up_down1 <- rbind(up_down1,c(as.character(dat1$time[ups[k]]),as.character(dat1$time[down1])))}
        }
      }
    }
    
    if(!is.null(up_down1)) get_baseMax(up_down1  , dat1[,s],dataset=dat1)->baseMax
    
    ########  get accurate mid point ########
    ########################################
    auc<-0
    if (!is.null(up_down1)) {
      up_down1<- cbind(up_down1, baseMax)
      # up_down1 <- cbind(up_down1,rep(mid.point0,nrow(up_down1)))
      colnames(up_down1) <- c('up.time','down.time','base.height','max.height')
      
      #### get AUC #########
      up_down.accu <- up_down1
      time.s <- chron(times=dat1$time)
      up.accu <- NULL
      down.accu <- NULL
      osc_len1.accu <- NULL
      
      for (t in 1:nrow(up_down1)) {
        if (t<nrow(up_down1)) start.next <- chron(times=up_down1[t+1,1]) else start.next <- min(chron(times=up_down1[t,2])+chron(times="00:05:00"), chron(times=time.s[length(time.s)])) 
        # if (t<nrow(up_down1)) base1<- up_down1[t+1, 3] else base1<- up_down1[t, 3] 
        base1<- as.numeric(up_down1[t, 3] )+(as.numeric(up_down1[t,4])-as.numeric(up_down1[t, 3] ))*0.18
        if (t>1) end.last <- chron(times=up_down1[t-1,2]) else end.last <- max(time.s[1],chron(times=up_down1[1,1])-chron(times="00:05:00"))
        time.range <- which(time.s>=end.last & time.s< start.next)
        above.base <- dat1[time.range,s]>base1
        
        longest.true <- function(torf,max.len=0) {
          if (length(torf)==1) {max.len <- max.len+1*torf; max.len} else {
            if (!torf[1]) max.len else  longest.true(torf[-1],max.len+1)}}
        
        
        runofT <- sapply(1:(length(above.base)-1),function(v) longest.true(above.base[-(1:v)]))
        if (sum(runofT)==0) runofT[1] <- T
        up_down.accu[t,1] <- as.character(dat1$time[time.range[which.max(runofT)]])
        up_down.accu[t,2] <- as.character(dat1$time[time.range[which.max(runofT)+max(runofT)-1]])
        up_down.accu[t,1] <-as.character(min(chron(time=up_down.accu[t,1]),chron(time=up_down1[t,1])))
        up_down.accu[t,2] <-as.character(max(chron(time=up_down.accu[t,2]),chron(time=up_down1[t,2])))
        
        up_down1[t,1] <- up_down.accu[t,1]
        up_down1[t,2] <- up_down.accu[t,2]
        
      }
      
      osc_len1.accu <- toMinute(chron(times=up_down1[1,2])-chron(times=up_down1[1,1]))
      
      ###
      step.min <- toMinute(chron(times=dat1$time)[2]-chron(times=dat1$time)[1])
      if (!is.null(up_down1)) {
        for (i in 1:nrow(up_down1)) {
          dat2 <- dat1[chron(times=dat1$time)>=chron(times=up_down1[i,1]) & chron(times=dat1$time)<=chron(times=up_down1[i,2]),]
          auc <- auc+sum(dat2[,s])
        }
        
      }
      auc <- round(auc*step.min,2)
      
    }
    auc.all <- auc.all +auc
    if (!is.null(up_down1)>0) up_down.all <- rbind(up_down.all,up_down1)
    
  }
  
  up_down1<- up_down.all
  nrow(up_down1)
  if (!is.null(up_down1)) n.osc <- nrow(up_down1) else n.osc <- 0
  osc_len1 <-NA
  delay1 <- NA
  
  n.osc<-0
  time.first.last<- NA
  osc.per.10min<- NA
  meanPeak<- NA
  TIbO<- NA
  
  n.osc1<-0
  time.first.last1<- NA
  osc.per.10min1<- NA
  meanPeak1<- NA
  TIbO1<- NA
  
  n.osc2<-0
  time.first.last2<- NA
  osc.per.10min2<- NA
  meanPeak2<- NA
  TIbO2<- NA
  if (!is.null(up_down1)) {
    ###### get summary stats ########
    osc_len1 <-NA
    delay1 <- NA
    
    n.osc<-0
    time.first.last<- NA
    osc.per.10min<- NA
    meanPeak<- NA
    TIbO<- NA
    
    n.osc1<-0
    time.first.last1<- NA
    osc.per.10min1<- NA
    meanPeak1<- NA
    TIbO1<- NA
    
    n.osc2<-0
    time.first.last2<- NA
    osc.per.10min2<- NA
    meanPeak2<- NA
    TIbO2<- NA
    
    if (!is.null(up_down1)){
      delay1 <- toMinute(up_down1[1,1])
      osc_len <- apply(up_down1,1,function(v){toMinute(chron(times=v[2])-chron(times=v[1]))})
      osc_len1 <- osc_len[1] 
      osc_len.rest <- mean(osc_len[-1])
      
      get_baseMax(up_down1  , dat[,s])->baseMax
      meanPeak <- mean(baseMax[,2]-baseMax[,1])
      ## all
      n.osc <- nrow(up_down1)
      
      if (n.osc >1) {
        time.first.last <-  toMinute(chron(times=up_down1[nrow(up_down1),2])-chron(times=up_down1[1,2]))
        osc.per.10min <-n.osc/time.first.last*10
        delays <- apply(cbind(up_down1[-nrow(up_down1),2],up_down1[-1,1]),1,function(v){toMinute(chron(times=v[2])-chron(times=v[1]))})
      } else {
        time.first.last <-0
        osc.per.10min <-n.osc/10
        delays <-NA
      }
      if (length(delays)>2)
        TIbO <- coef(lm(1:length(delays)~delays))[2]
      
      ## 1 hour after 1st osc
      up_down2 <- up_down1[which( hours(chron(times=up_down1[,1])-chron(times=up_down1[1,1]))==0),,drop=F]
      n.osc1 <- nrow(up_down2)
      
      if (n.osc1>0) {get_baseMax(up_down2  , dat[,s])->baseMax1
        meanPeak1 <- mean(baseMax1[,2]-baseMax1[,1])
        
        if (n.osc1 >1) {
          time.first.last1 <-  toMinute(chron(times=up_down2[nrow(up_down2),2])-chron(times=up_down2[1,2]))
          osc.per.10min1 <-n.osc1/time.first.last1*10
          delays1 <- apply(cbind(up_down2[-nrow(up_down2),2],up_down2[-1,1]),1,function(v){toMinute(chron(times=v[2])-chron(times=v[1]))})
        }
       else {
        time.first.last1 <-0
        osc.per.10min1 <-n.osc/10
        delays1 <-NA
      }
      }
      if (length(delays1)>2)
        TIbO1 <- coef(lm(1:length(delays1)~delays1))[2]
      
      ## 2 hours
      up_down2 <- up_down1[which( hours(chron(times=up_down1[,1])-chron(times=up_down1[1,1]))<=1),,drop=F]
      n.osc2 <- nrow(up_down2)
      
      if (n.osc2>0) {get_baseMax(up_down2  , dat[,s])->baseMax2
        meanPeak2 <- mean(baseMax2[,2]-baseMax2[,1])
        
        if (n.osc2 >1) {
          time.first.last2 <-  toMinute(chron(times=up_down2[nrow(up_down2),2])-chron(times=up_down2[1,2]))
          osc.per.10min2 <-n.osc2/time.first.last2*10
          delays2 <- apply(cbind(up_down2[-nrow(up_down2),2],up_down2[-1,1]),1,function(v){toMinute(chron(times=v[2])-chron(times=v[1]))})
        }  else {
          time.first.last2 <-0
          osc.per.10min2 <-n.osc/10
          delays2 <-NA
        }}
      
      if (length(delays2)>2)
        TIbO2 <- coef(lm(1:length(delays2)~delays2))[2]}
    
    
    
    if (!is.null(up_down1)) {
      mid.time <- (chron(times=up_down1[,2])+chron(times=up_down1[,1]))/2
      up_down1<- cbind(up_down1,as.character(mid.time))
      colnames(up_down1)[5] <- 'mid.time'
     # write.table(up_down1,row.names = F,quote=F,sep='\t',file=paste('sample',s-2,'.txt',sep=''))
      }
    
    
    if (!is.na(length.point)) {
      time.across <- which(sapply(2:nrow(dat),function(v) { (dat[,s]<length.at.height)[v]!=(dat[,s]<length.at.height)[v-1]}))
      time.across <- (time.across[time.across>=which(dat$time==up_down1[1,1]) & time.across<=which(dat$time==up_down1[1,2])])
      if (length(time.across)==0) osc_len1.accu <- 0 else  osc_len1.accu <- toMinute(chron(times= dat$time[range(time.across)[2]])-chron(times=dat$time[range(time.across)[1]]))
    }
    
    #### get areas under the peaks ###
    auc <- 0
    step.min <- toMinute(chron(times=dat$time)[2]-chron(times=dat$time)[1])
    if (!is.null(up_down1)) {
      for (i in 1:nrow(up_down1)) {
        dat1 <- dat[chron(times=dat$time)>=chron(times=up_down1[i,1]) & chron(times=dat$time)<=chron(times=up_down1[i,2]),]
        auc <- auc+sum(pmax(dat1[,s]-as.numeric(up_down1[i,3]),rep(0,nrow(dat1))))
      }
      
    }
    auc <- round(auc*step.min,2)
    
    if (!is.null(up_down1)) auc0 <- round(get.auc(s1=s,up_down=up_down1,lag.time=lag.time1,first.peak=F)*step.min,2) else auc0<-0
    if (!is.null(up_down1)) auc1 <- round(get.auc(s1=s,up_down=up_down1,lag.time=lag.time1,first.peak=T)*step.min,2) else auc1<-0
    if(save.plot) {pdf('plot.onesample.pdf')}
    ######### plotting ######## 
    plot(dat$time,dat[,s],type='l',xlab='time',ylab='value',main=paste('Sample',s-2,' # of Osc=',n.osc,'; 1st length=',round(osc_len1.accu,2),' AUC=',auc," & ",auc0," & ",auc1,sep=''))
    #   mtext('|',side=1,at=chron(times=c('00:30:00','01:30:00','02:30:00','03:30:00')))
    #   mtext(c('00:30','01:30','02:30','00:30'),line=1,side=1,at=chron(times=c('00:30:00','01:30:00','02:30:00','03:30:00')))
    abline(v=chron(times=up_down1[,5]),lty=2)
    if (!is.na(at.time[1])) abline(v=chron(times=at.time),lty=1,col='red') ### blue to show where the segments 
    #abline(v=chron(times=up_down1[1,2]),lty=2)
    if (length(mid.point0)==1) abline(h=mid.point0,col='red') else
      for (seg in 1:length(mid.point0)){
        if (seg==length(mid.point0))
          segments(chron(times=at.time[seg-1]),mid.point0[seg],max(dat$time),mid.point0[seg],col='red') else if (seg==1) 
            segments(dat$time[1],mid.point0[1],chron(times=at.time[1]),mid.point0[1],col='red') else
              segments(chron(times=at.time[seg-1]),mid.point0[seg],chron(times=at.time[seg]),mid.point0[seg],col='red')
        
      }
    if (!is.na(length.point)) segments(chron(times= dat$time[range(time.across)[1]]),length.point,chron(times=dat$time[range(time.across)[2]]),length.point,col='blue',lwd=2)
    
    abline(v=chron(times=up_down1[1,1])+chron(times=lag.time1),col='blue')
    paint.auc(up_down1,s1=s, lag.time=lag.time1,first.peak=T)
    points(dat$time,dat[,s],type='l')
    
    
    rst <- cbind(rst,c(osc_len1.accu ,auc,auc1, auc0,delay1,n.osc,time.first.last,osc.per.10min,meanPeak,
                       n.osc1,time.first.last1,osc.per.10min1,meanPeak1,
                       n.osc2,time.first.last2,osc.per.10min2,meanPeak2))
    rownames(rst) <- c("length of 1st Osc",'Total AUC','AUC w 1st peak','AUC w/o 1st peak',"Delay b/f 1st",
                       "n.osc (all)","time.first.last (all)","osc.per.10min (all)","meanPeak (all)",
                       "n.osc (1hr)","time.first.last (1hr)","osc.per.10min (1hr)","meanPeak (1hr)",
                       "n.osc (2hr)","time.first.last (2hr)","osc.per.10min (2hr)","meanPeak (2hr)")
    colnames(rst)<-paste('Sample',s-2)
    print(rst)
    write.table(rst,col.names = NA,sep='\t',file=file1,quote=F,append=.append)
    
  } else plot(dat$time,dat[,s],type='l',xlab='time',ylab='value',main=paste('Sample',s-2,' # of Osc=',n.osc,sep=''))
  
  if (save.plot) dev.off()
}

#----------------------------------------------------------------------#
setwd('#---------------- your-directory -----------------#')


read.table('example.txt',header=T,as.is=T)->dat
dat$time <- chron(times=dat$time)

###------------------   run sample by sample within dat and save output to files -----------------------###
pdf('all.plot.pdf')
rst<- NA
for (s in 3:ncol(dat)-2){
  if (is.na(rst)) rst <-count.oscillation(dat,s,save.plot = F) else rst<- cbind(rst,count.oscillation(dat,s,save.plot = F))
}
colnames(rst) <- paste('sample',1:(ncol(dat)-2),sep='_')
write.table(rst,row.names = F,quote=F,sep='\t',file='all_stats.txt')
dev.off()
###-----------------------------------------------------------------------------------------------------###


count.oscillation(dat, s=1,save.plot = T)
count.osc.drawline(3,mid.point0=c(2.15,2.2,2.15),at.time = c('01:30:00','02:00:56'),length.at.height=2)

